'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _mobx = require('mobx');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _jss = require('react-jss/lib/jss');

var _server = require('react-universal-component/server');

var _PWA = require('./PWA');

var _PWA2 = _interopRequireDefault(_PWA);

var _createMemoryHistory = require('history/createMemoryHistory');

var _createMemoryHistory2 = _interopRequireDefault(_createMemoryHistory);

var _reactHelmet = require('react-helmet');

var _sanitizeAmpHtml = require('./amp/sanitizeAmpHtml');

var _sanitizeAmpHtml2 = _interopRequireDefault(_sanitizeAmpHtml);

var _renderers = require('./renderers');

var _Response = require('./router/Response');

var _Response2 = _interopRequireDefault(_Response);

var _Track = require('./Track');

var _createRequest = require('./platform/createRequest');

var _createRequest2 = _interopRequireDefault(_createRequest);

var _reactStorefrontStats = require('react-storefront-stats');

var _reactStorefrontStats2 = _interopRequireDefault(_reactStorefrontStats);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _mobx.configure)({ isolateGlobalState: true }); /**
                                                     * @license
                                                     * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
                                                     */

var Server = function () {

  /**
   * @param {Object} config
   * @param {Object} config.theme A material-UI theme
   * @param {Class} config.model The model class for the root of the state tree
   * @param {React.Component} config.App The root app component
   * @param {Router} config.router An instance of moov_router's Router class
   * @param {Boolean} [config.deferScripts=true] Adds the defer attribute to all script tags to speed up initial page render. Defaults to true.
   */
  function Server(_ref) {
    var _this = this;

    var theme = _ref.theme,
        model = _ref.model,
        App = _ref.App,
        router = _ref.router,
        _ref$deferScripts = _ref.deferScripts,
        deferScripts = _ref$deferScripts === undefined ? true : _ref$deferScripts;
    (0, _classCallCheck3.default)(this, Server);
    this.serve = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      var request, response, state;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              console.error = console.error || console.log;
              console.warn = console.warn || console.log;

              request = (0, _createRequest2.default)();
              response = new _Response2.default(request);
              _context.prev = 4;

              // indicate to the XDN that we want to to add set the x-moov-cache-hit cookie so we can differentiate
              // cache hits and misses when tracking performance
              response.set('x-rsf-track-cache-hit', 'true');

              _context.next = 8;
              return _this.router.runAll(request, response);

            case 8:
              state = _context.sent;

              if (!(!state.proxyUpstream && !response.sent)) {
                _context.next = 12;
                break;
              }

              _context.next = 12;
              return _this.renderPWA({ request: request, response: response, state: state });

            case 12:
              _context.next = 18;
              break;

            case 14:
              _context.prev = 14;
              _context.t0 = _context['catch'](4);
              _context.next = 18;
              return _this.renderError(_context.t0, request, response);

            case 18:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[4, 14]]);
    }));

    console.error = console.warn = console.log;

    Object.assign(this, {
      theme: theme,
      model: model,
      App: App,
      router: router,
      deferScripts: deferScripts
    });
  }

  /**
   * Handles an isomorphic request by serving json, html, or amp content based on the URL.
   */


  (0, _createClass3.default)(Server, [{
    key: 'setContentType',


    /**
     * Sets the content type to application/json for json URLs, text/html for all others
     * @param {Object} request 
     * @param {Response} response 
     */
    value: function setContentType(request, response) {
      if (response.get('content-type') == null) {
        if (request.pathname.endsWith('.json')) {
          response.set('content-type', 'application/json');
        } else {
          response.set('content-type', 'text/html');
        }
      }
    }

    /**
     * Renders either a JSON or HTML response for the given state based on the path suffix.
     * @param {Object} options
     * @param {Object} options.request The current request object
     * @param {Response} options.response The current response object
     * @param {Object} options.state The root react element
     * @return The html for app
     */

  }, {
    key: 'renderPWA',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref4) {
        var request = _ref4.request,
            response = _ref4.response,
            state = _ref4.state;
        var protocol, hostname, port, pathname, search, amp, App, theme, sheetsRegistry, history, model, stats, html, helmet;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                console.error = console.error || console.log;
                console.warn = console.warn || console.log;

                protocol = request.protocol, hostname = request.hostname, port = request.port, pathname = request.pathname, search = request.search;

                this.setContentType(request, response);

                if (!pathname.endsWith('.json')) {
                  _context2.next = 6;
                  break;
                }

                return _context2.abrupt('return', response.send(JSON.stringify(state)));

              case 6:
                amp = pathname.endsWith('.amp');
                App = this.App, theme = this.theme;
                sheetsRegistry = new _jss.SheetsRegistry();
                history = (0, _createMemoryHistory2.default)({ initialEntries: [pathname + search] });
                model = this.model.create((0, _extends3.default)({}, state, {
                  amp: amp,
                  initialWidth: amp ? 'xs' : state.initialWidth,
                  location: (0, _extends3.default)({}, history.location, {
                    port: port,
                    protocol: protocol,
                    hostname: hostname
                  })
                }));
                _context2.prev = 11;
                _context2.next = 14;
                return (0, _reactStorefrontStats2.default)();

              case 14:
                stats = _context2.sent;
                html = (0, _renderers.renderHtml)({
                  component: _react2.default.createElement(
                    _PWA2.default,
                    null,
                    _react2.default.createElement(App, null)
                  ),
                  providers: {
                    app: model,
                    history: history,
                    analytics: {},
                    router: this.router
                  },
                  registry: sheetsRegistry,
                  theme: theme
                });
                helmet = _reactHelmet.Helmet.renderStatic();


                html = '\n        <!DOCTYPE html>\n        <html' + (amp ? ' ⚡' : '') + ' ' + helmet.htmlAttributes.toString() + '>\n          <head>\n            ' + helmet.title.toString() + '\n            ' + (amp ? '\n              <style amp-boilerplate>body{-webkit-animation:-amp-start 8s steps(1,end) 0s 1 normal both;-moz-animation:-amp-start 8s steps(1,end) 0s 1 normal both;-ms-animation:-amp-start 8s steps(1,end) 0s 1 normal both;animation:-amp-start 8s steps(1,end) 0s 1 normal both}@-webkit-keyframes -amp-start{from{visibility:hidden}to{visibility:visible}}@-moz-keyframes -amp-start{from{visibility:hidden}to{visibility:visible}}@-ms-keyframes -amp-start{from{visibility:hidden}to{visibility:visible}}@-o-keyframes -amp-start{from{visibility:hidden}to{visibility:visible}}@keyframes -amp-start{from{visibility:hidden}to{visibility:visible}}</style><noscript><style amp-boilerplate>body{-webkit-animation:none;-moz-animation:none;-ms-animation:none;animation:none}</style></noscript>\n              <style amp-custom id="ssr-css">\n                ' + sheetsRegistry.toString().replace(/\!important/g, '') + '\n                ' + helmet.style.toString().replace(/<\/?style[^>]*>/g, '') + '\n              </style>\n            ' : '\n              <noscript id="jss-insertion-point"></noscript>\n            ') + '\n            ' + helmet.meta.toString() + '\n            ' + helmet.link.toString() + '\n            ' + helmet.script.toString() + '\n            ' + (amp ? '<script async custom-element="amp-analytics" src="https://cdn.ampproject.org/v0/amp-analytics-0.1.js"></script>' : '') + '\n          </head>\n          <body' + (amp ? ' class="moov-amp"' : '') + ' ' + helmet.bodyAttributes.toString() + '>\n            ' + (amp ? (0, _Track.renderAmpAnalyticsTags)() : '') + '\n            ' + (amp ? '' : (0, _renderers.renderStyle)({ registry: sheetsRegistry })) + '\n            <noscript>\n              You need to enable JavaScript to run this app.\n            </noscript>\n            <div id="root">' + html + '</div>\n            ' + (amp ? '' : '\n              ' + (0, _renderers.renderInitialStateScript)({ state: model, defer: this.deferScripts }) + '\n              ' + (0, _renderers.renderScript)({ stats: stats, chunk: 'bootstrap', defer: this.deferScripts }) + '\n              ' + this.getScripts(stats) + '\n              ' + (0, _renderers.renderScript)({ stats: stats, chunk: 'main', defer: this.deferScripts }) + '\n            ') + '\n          </body>\n        </html>\n      ';

                if (amp) {
                  html = (0, _sanitizeAmpHtml2.default)(html);
                }

                response.send(html);
                _context2.next = 27;
                break;

              case 22:
                _context2.prev = 22;
                _context2.t0 = _context2['catch'](11);

                // flush head content to prevent memory leak, see https://github.com/nfl/react-helmet#server-usage
                _reactHelmet.Helmet.renderStatic();

                // flush amp analytics tags to prevent memory leak
                (0, _Track.renderAmpAnalyticsTags)();

                throw _context2.t0;

              case 27:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[11, 22]]);
      }));

      function renderPWA(_x) {
        return _ref3.apply(this, arguments);
      }

      return renderPWA;
    }()

    /**
     * Renders an error response, either as JSON or SSR HTML, depending on the suffix
     * on the request path.
     * @param {Error} e 
     * @param {Response} response 
     */

  }, {
    key: 'renderError',
    value: function renderError(e, request, response) {
      response.status(500, 'error');

      return this.renderPWA({
        request: request,
        response: response,
        state: {
          page: 'Error',
          error: e.message,
          stack: process.env.MOOV_ENV === 'production' ? null : e.stack
        }
      });
    }

    /**
     * Gets the script tags that should added to the document based on the chunks used
     * to render the current request.
     * @param {Object} stats 
     * @return {String[]}
     */

  }, {
    key: 'getScripts',
    value: function getScripts(stats) {
      var _this2 = this;

      return (0, _server.flushChunkNames)().map(function (chunk) {
        return (0, _renderers.renderScript)({ stats: stats, chunk: chunk, defer: _this2.deferScripts });
      }).filter(function (e) {
        return !!e;
      });
    }
  }]);
  return Server;
}();

exports.default = Server;
//# sourceMappingURL=Server.js.map