'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SortBase = exports.FacetGroupModelBase = exports.FacetModelBase = exports.LAYOUT_GRID = exports.LAYOUT_LIST = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _mobxStateTree = require('mobx-state-tree');

var _fetch = require('fetch');

var _fetch2 = _interopRequireDefault(_fetch);

var _ProductModelBase = require('./ProductModelBase');

var _ProductModelBase2 = _interopRequireDefault(_ProductModelBase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LAYOUT_LIST = exports.LAYOUT_LIST = 'LAYOUT_LIST'; /**
                                                        * @license
                                                        * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
                                                        */
var LAYOUT_GRID = exports.LAYOUT_GRID = 'LAYOUT_GRID';

/**
 * A search facet that can be applied as a filter
 */
var FacetModelBase = exports.FacetModelBase = _mobxStateTree.types.model('FacetModelBase', {
  /**
   * The value sent to the API when filtering
   */
  code: _mobxStateTree.types.maybe(_mobxStateTree.types.string),

  /**
   * The text displayed in the UI
   */
  name: _mobxStateTree.types.identifier(_mobxStateTree.types.string),

  /**
   * The number of items having the value
   */
  matches: _mobxStateTree.types.maybe(_mobxStateTree.types.number)
});

/**
 * A grouping of facets
 */
var FacetGroupModelBase = exports.FacetGroupModelBase = _mobxStateTree.types.model('FacetGroupModelBase', {
  /**
   * The text displayed in the UI
   */
  name: _mobxStateTree.types.identifier(_mobxStateTree.types.string),

  /**
   * The facets in the group
   */
  facets: _mobxStateTree.types.optional(_mobxStateTree.types.array(FacetModelBase), [])
});

/**
 * A value by which results can be sorted
 */
var SortBase = exports.SortBase = _mobxStateTree.types.model('SortBase', {
  /**
   * The code sent to the API when sorting
   */
  code: _mobxStateTree.types.identifier(_mobxStateTree.types.string),

  /**
   * The text displayed in the UI
   */
  name: _mobxStateTree.types.maybe(_mobxStateTree.types.string)
});

/**
 * A base class for search results and subcategories that supports filtering and sorting
 */
exports.default = _mobxStateTree.types.model('SearchResultsModelBase', {
  /**
   * The codes of the currently applied filters
   */
  filters: _mobxStateTree.types.optional(_mobxStateTree.types.array(_mobxStateTree.types.string), []),

  /**
   * Automatically set to true when the user has changed a filter.  This causes
   * the filter dialog footer to be displayed.
   */
  filtersChanged: false,

  /**
   * Filters that can be selected
   */
  facetGroups: _mobxStateTree.types.optional(_mobxStateTree.types.array(FacetGroupModelBase), []),

  /**
   * The code for the currently selected sort option
   */
  sort: _mobxStateTree.types.maybe(_mobxStateTree.types.string),

  /**
   * Sort options that can be selected
   */
  sortOptions: _mobxStateTree.types.optional(_mobxStateTree.types.array(SortBase), []),

  /**
   * The total number of matching items. You can either specify numberOfPages or total to determine
   * when to display the Show More Button.  Only one is required.
   */
  total: _mobxStateTree.types.maybe(_mobxStateTree.types.number),

  /**
   * The total number of pages.  You can either specify numberOfPages or total to determine
   * when to display the Show More Button.  Only one is required.
   */
  numberOfPages: _mobxStateTree.types.maybe(_mobxStateTree.types.number),

  /**
   * The matching items
   */
  items: _mobxStateTree.types.optional(_mobxStateTree.types.array(_ProductModelBase2.default), []),

  /**
   * Set to true when fetching data from the server in response to a filter or sort change.
   */
  loading: false,

  /**
   * Set to true when loading more items from the server is in progress.
   */
  loadingMore: false,

  /**
   * The current page being displayed
   */
  page: 0,

  /**
   * The maximum number of records per page
   */
  pageSize: 10,

  /**
   * Sets the layout style on the view.  Can be LAYOUT_LIST or LAYOUT_GRID.  Defaults to LAYOUT_GRID.
   */
  layout: _mobxStateTree.types.optional(_mobxStateTree.types.enumeration('Layout', [LAYOUT_LIST, LAYOUT_GRID]), LAYOUT_GRID)
}).views(function (self) {
  return {
    get hasMoreItems() {
      if (self.numberOfPages != null) {
        return self.numberOfPages - 1 > self.page;
      } else if (self.total != null) {
        return self.total > self.items.length;
      } else {
        return false;
      }
    }
  };
}).actions(function (self) {
  return {
    /**
     * Removes all filters.
     */
    clearAllFilters: function clearAllFilters() {
      self.filters.clear();
      self.filtersChanged = true;
    },

    /**
     * Sets the filtersChanged field
     * @param {Boolean} changed
     */
    setFiltersChanged: function setFiltersChanged(changed) {
      self.filtersChanged = changed;
    },

    /**
     * Refreshes the search based on the current filters and sort and resets the page to 0.
     */
    refresh: (0, _mobxStateTree.flow)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              self.filtersChanged = false;
              self.loading = true;
              _context.next = 4;
              return self.showPage(0);

            case 4:
              self.loading = false;

            case 5:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    })),
    /**
     * Toggles a facet on or off.
     * @param {FacetModelBase} facet
     */
    toggleFilter: function toggleFilter(facet) {
      var code = facet.code;

      var index = self.filters.indexOf(code);

      if (index === -1) {
        self.filters.push(code);
      } else {
        self.filters.splice(index, 1);
      }
      self.filtersChanged = true;
    },

    showPage: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(page) {
        var _window$location, pathname, search, results;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                self.page = page;
                _window$location = window.location, pathname = _window$location.pathname, search = _window$location.search;

                search += (search.length ? '&' : '?') + 'page=' + self.page;

                _context2.prev = 3;

                self.loadingMore = page > 0;
                _context2.next = 7;
                return (0, _fetch2.default)(self.getShowMoreURL(pathname + '.json' + search)).then(function (res) {
                  return res.json();
                });

              case 7:
                results = _context2.sent;

                if ((0, _mobxStateTree.isAlive)(self)) {
                  self.addItems(results.items);
                  self.setTotal(results.total);
                }

              case 9:
                _context2.prev = 9;

                if ((0, _mobxStateTree.isAlive)(self)) {
                  self.endLoadingMore();
                }
                return _context2.finish(9);

              case 12:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[3,, 9, 12]]);
      }));

      function showPage(_x) {
        return _ref.apply(this, arguments);
      }

      return showPage;
    }(),
    /**
     * Fetches the next page from the server.
     */
    showMore: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (self.loading) {
                  _context3.next = 3;
                  break;
                }

                _context3.next = 3;
                return self.showPage(self.page + 1);

              case 3:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function showMore() {
        return _ref2.apply(this, arguments);
      }

      return showMore;
    }(),
    getShowMoreURL: function getShowMoreURL(defaultURL) {
      return defaultURL;
    },

    /**
     * Hides the loading more spinner.
     */
    endLoadingMore: function endLoadingMore() {
      self.loadingMore = false;
    },

    /**
     * Adds more items to the results
     * @param {Array} items
     */
    addItems: function addItems(items) {
      if (self.page === 0) {
        self.items = [];
      }
      if (Array.isArray(items)) {
        self.items = [].concat((0, _toConsumableArray3.default)(self.items), (0, _toConsumableArray3.default)(items));
      }
      self.endLoadingMore();
    },

    /**
     * Sets the total number of matching records
     * @param {Integer} total 
     */
    setTotal: function setTotal(total) {
      self.total = total;
    },

    /**
     * Toggles the layout
     * @param {String} layout LAYOUT_LIST or LAYOUT_GRID
     */
    switchLayout: function switchLayout(layout) {
      self.layout = layout;
    },

    /**
     * Sets the selected sort option
     */
    setSort: function setSort(option) {
      self.sort = option.code;
    }
  };
});
//# sourceMappingURL=SearchResultsModelBase.js.map